
-- a simple bar creator. credits to Freebaser and his cool addon: ikon.

local myname, ns = ...

local f = CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...) 
	if ns[event] then 
		return ns[event](ns, event, ...) 
	end 
end)

function ns:RegisterEvent(...) 
	for i=1,select("#", ...) do 
		f:RegisterEvent((select(i, ...))) 
	end 
end

function ns:UnregisterEvent(...) 
	for i=1,select("#", ...) 
		do f:UnregisterEvent((select(i, ...))) 
	end 
end

ns.RegisterEvents, ns.UnregisterEvents = ns.RegisterEvent, ns.UnregisterEvent

ns.icons = {}
local db, cds, auras, debuffs, items
local CD, AURA, DEBUFF, ITEM = "CD", "AURA", "DEBUFF", "ITEM"

local function findIcon(name, type)
	for i=#(ns.icons), 1, -1 do
		if ns.icons[i].name == name and ns.icons[i].type == type then
			local frame = ns.icons[i]
			return frame
		end
	end
end

local function UnregIcon(frame, name, type)
	if frame then
		frame:Hide()
		frame.update = true
	elseif name then
		local f = findIcon(name, type)
		return UnregIcon(f)
	end
end

local fmod = math.fmod

local function OnUpdate()
	return function(self, elapsed)
		local duration = self.duration - elapsed
		if duration <= 0 then
			UnregIcon(self)
			return
		end
		self.duration = duration
		local sb = self.sb
		if self.reverse == true then
			local percent = duration / self.max
			if self.vertical == true then
				sb:SetPoint("TOP", 0, (self.height - self.width) * (1 - percent) - self.width)
			else
				sb:SetPoint("LEFT", (self.width - self.height) * (1 - percent), 0)
			end
		end		
		sb:SetValue(duration)
		
		local min, sec = floor(duration / 60), fmod(duration, 60)
		
		if(min > 0) then
			self.timer:SetFormattedText("%d:%02d", min, sec)
		elseif(sec < 10) then
			self.timer:SetFormattedText("%.1f", sec)
		else
			self.timer:SetFormattedText("%d", sec)
		end
		
	end
end

local function CreateIcon(name, obj, type)
	local frame = CreateFrame("Frame", name.."ikon", UIParent)
	frame:SetBackdrop({bgFile = "Interface\\Addons\\Freebars\\media\\flat";})
	frame:SetBackdropColor(backdrop_r, backdrop_g, backdrop_b, backdrop_t)	
	frame:SetFrameStrata"BACKGROUND"
	frame:SetPoint("TOP", UIParent, "CENTER", obj.xpos, obj.ypos)
	frame:Hide()

	local icon = frame:CreateTexture(nil, "OVERLAY")
	icon:SetTexCoord(.07, .93, .07, .93)

	local sb = CreateFrame"StatusBar"
	sb:SetParent(frame)
	sb:SetStatusBarTexture("Interface\\Addons\\Freebars\\media\\flat")
	
	if obj.vertical ~= true then
		dump = obj.height
		obj.height = obj.width
		obj.width = dump
	end
	
	frame:SetSize(obj.width, obj.height)
	if obj.vertical == true then
		icon:SetSize(obj.width, obj.width)
		sb:SetSize(obj.width, obj.height-obj.width)
		sb:SetOrientation("VERTICAL")
		if obj.reverse == true then
			icon:SetPoint("TOP")
		else
			icon:SetPoint("BOTTOM")
			sb:SetPoint("BOTTOM", 0, obj.width)
		end		
	else
		icon:SetSize(obj.height, obj.height)
		sb:SetSize(obj.width-obj.height, obj.height)
		if obj.reverse == true then
			icon:SetPoint("RIGHT")
		else
			icon:SetPoint("LEFT")
			sb:SetPoint("LEFT", obj.height, 0)
		end
	end
	
	local count = frame:CreateFontString(nil, "OVERLAY")
	count:SetFont(font, obj.countsize, fonttype)
	count:SetTextColor(1, 1, 1)
	count:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", -1, 1)
	if obj.countsize == 1 then count:Hide() end
	
	local timer = sb:CreateFontString(nil, "OVERLAY")
	timer:SetFont(font, obj.timersize, fonttype)
	timer:SetTextColor(1, 1, 1)
	timer:SetPoint("CENTER", icon, "CENTER", 0, 0)
	if obj.timersize == 1 then timer:Hide() end
	
	frame:SetScript("OnUpdate", OnUpdate())
	frame.icon = icon
	frame.sb = sb
	frame.name = tostring(name)
	frame.type = type
	frame.count = count
	frame.timer = timer
	frame.r = obj.r
	frame.g = obj.g
	frame.b = obj.b
	frame.width = obj.width
	frame.height = obj.height
	
	frame.reverse = obj.reverse
	frame.vertical = obj.vertical

	table.insert(ns.icons, frame)
end

local function RegIcon(name, startTime, seconds, dura, icon, count, type)
	local frame = findIcon(name, type)
	if frame == nil then return end
	if count and count > 0 then
		frame.count:SetText(count)
	end
	if frame.update == false then
		if frame.duration and seconds < frame.duration + 0.5 then return end
	end
	frame.update = false
	local duration = startTime - GetTime() + seconds
	frame.duration = duration
	frame.max = dura
	local sb = frame.sb
	sb:SetStatusBarColor(frame.r, frame.g, frame.b)
	sb:SetMinMaxValues(0, dura)
	sb:SetValue(duration)
	frame.icon:SetTexture(icon)
	frame:Show()
end

ns:RegisterEvent("SPELL_UPDATE_COOLDOWN")
function ns:SPELL_UPDATE_COOLDOWN()
	for cd, obj in pairs(cds) do
		local startTime, duration, enabled = GetSpellCooldown(cd)
		if(enabled == 1 and duration > 1.5) then
				RegIcon(cd, startTime, duration, duration, GetSpellTexture(cd), nil, CD)
		end
	end
end

ns:RegisterEvent("PLAYER_TARGET_CHANGED")
ns:RegisterEvent("UNIT_AURA")
function ns:UNIT_AURA()
	for aura, obj in pairs(auras) do 
		local name ,_, icon, count,_, duration, expires, caster,_,_, 
			spellID = UnitAura("player", aura, nil, "HELPFUL")
		if name then
			local startTime = GetTime()
			local secs = -(GetTime() - expires)
			RegIcon(aura, startTime, secs, duration, icon, count, AURA)
		else
			UnregIcon(nil, aura, AURA)
		end
	end
	for debuff, obj in pairs(debuffs) do 
		local name ,_, icon, count,_, duration, expires, caster,_,_, 
			spellID = UnitAura("target", debuff, nil, "HARMFUL")
		if name then
			if((not obj.player) or (obj.player and caster == "player")) then
				local startTime = GetTime()
				local secs = -(GetTime() - expires)
				RegIcon(debuff, startTime, secs, duration, icon, count, DEBUFF)
			end
		else
			UnregIcon(nil, debuff, DEBUFF)
		end
	end
end
ns.PLAYER_TARGET_CHANGED = ns.UNIT_AURA

ns:RegisterEvent("BAG_UPDATE_COOLDOWN")
function ns:BAG_UPDATE_COOLDOWN()
	for item, obj in pairs(items) do
		local startTime, duration, enabled = GetItemCooldown(item)
		if(enabled == 1) then
			RegIcon(tostring(item), startTime, duration, duration, select(10, GetItemInfo(item)), nil, ITEM)
		end
	end
end

local function GetIcons()
	for cd, obj in pairs(cds) do
		CreateIcon(cd, obj, CD)
	end
	for aura, obj in pairs(auras) do
		CreateIcon(aura, obj, AURA)
	end
	for debuff, obj in pairs(debuffs) do
		CreateIcon(debuff, obj, DEBUFF)
	end
	for item, obj in pairs(items) do
		CreateIcon(item, obj, ITEM)
	end
end

ns:RegisterEvent("PLAYER_LOGIN")
function ns:PLAYER_LOGIN()
	local _, class = UnitClass("player")
	db = ns.cfg[class] or nil
	if db then
		cds = db.cds or {}
		auras = db.auras or {}
		debuffs = db.debuffs or {}
		items = db.items or {}
		GetIcons()
	else
		ns:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
		ns:UnregisterEvent("UNIT_AURA")
		ns:UnregisterEvent("PLAYER_TARGET_CHANGED")
		ns:UnregisterEvent("BAG_UPDATE_COOLDOWN")
	end
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

